/* My 3-D dialog interface library
   interfaces to CTL3D.DLL or CTL3DV2.DLL and provides nice 3D effects for dialogs
   created using MFC V1.5 or greater
   
   Has two great features:
   	(a)	If the .DLL's are not installed the dialogs will revert to standard
   		without any error messages or dramatics
   	(b)	The use of the 3-D effects can be turned off and on at will while the
   		application is running by using Ctl3dRegister() and Ctl3dUnRegister()
   
   To use:
   
   1/	Add this file to the project and compile (or just link the object file)
   
   2/	Include myctl3d.h in any files which will call the following routines.
   
   3/	To enable the 3-D stuff call Ctl3dRegister()
   		The best place is in  CMyApp::InitInstance()
   
   4/   (optional)
   		In  CMainWnd::OnSysColorChange()  call  Ctl3dColorChange()
   		
   5/	To disable the 3-D stuff call Ctl3dUnRegister()
   		The best place is in  CMyApp::ExitInstance()
   		
   6/	If any common dialog boxes are used you should insert the address
   		of  OpenHook() into the hook member (see the function description for details)
*/

#include "stdafx.h"


// pointers to functions in the CTL3D.DLL (if loaded)
#define CTL3D_ALL				0xffff
HINSTANCE hinstCtl3D ;
BOOL (WINAPI *lpfnCtl3dSubclassDlg) (HWND hwndDlg, WORD grbit) ;
BOOL (WINAPI *lpfnCtl3dColorChange) (VOID) ;
BOOL (FAR PASCAL *lpfnCtl3dUnRegister) (HANDLE hinstApp) ;





///////////////////////////////////////////////////////////////////////////////////////////////
/* Register and setup all the 3-D stuff
   place a call to this function in the application InitInstance
   eg:
		BOOL CMyApp::InitInstance()
		{
			// create the main window
		    pMainWnd = m_pMainWnd = new CMainWnd();
		    m_pMainWnd->ShowWindow( m_nCmdShow );
		    m_pMainWnd->UpdateWindow();
		    		-
		        	-
			Ctl3dRegister() ;
					-
					-
		}
*/
void Ctl3dRegister(void)
{
	int ErrMode ;
	BOOL (WINAPI *lpfnCtl3dRegister) (HANDLE hinstApp) ;
	BOOL (WINAPI *lpfnCtl3dAutoSubclass) (HANDLE hinstApp) ;
	
	// skip 3D stuff if greater than ver 3.11 (ie. Windows95+)
	if((GetVersion() & 0xff) != 3) return ;			// check major (must be ver 3)
	if(((GetVersion() >> 8) & 0xff) > 11) return ;	// minor must be > ver 3.11
	
	hinstCtl3D = NULL ;
    // load the 3-D style dialog box library and link to the required routines
    // if this fails (library not found) then the user gets ordinary dialog boxes
	ErrMode = SetErrorMode(SEM_NOOPENFILEERRORBOX) ; // Turn off the "File not found" error box
	hinstCtl3D = LoadLibrary("CTL3D.DLL") ;          // Load the CTL3D.DLL library module
	if (hinstCtl3D <= HINSTANCE_ERROR)
		hinstCtl3D = LoadLibrary("CTL3DV2.DLL") ;    // If that did not work try CTL3DV2.DLL
	if (hinstCtl3D <= HINSTANCE_ERROR)
		return ;                                     // If that did not work then give up!

    // successfully loaded the library, next get the function addresses
    lpfnCtl3dRegister = (BOOL (WINAPI *) (HANDLE))GetProcAddress(hinstCtl3D, "Ctl3dRegister") ;
    lpfnCtl3dAutoSubclass = (BOOL (WINAPI *) (HANDLE))GetProcAddress(hinstCtl3D, "Ctl3dAutoSubclass") ;
    lpfnCtl3dUnRegister = (BOOL (WINAPI *) (HANDLE))GetProcAddress(hinstCtl3D, "Ctl3dUnRegister") ;
    lpfnCtl3dSubclassDlg = (BOOL (WINAPI *) (HWND, WORD))GetProcAddress(hinstCtl3D, "Ctl3dSubclassDlg") ;
    lpfnCtl3dColorChange = (BOOL (WINAPI *) (VOID))GetProcAddress(hinstCtl3D, "Ctl3dColorChange") ;
    
    if(lpfnCtl3dRegister == NULL || lpfnCtl3dAutoSubclass == NULL || lpfnCtl3dUnRegister == NULL 
    							 || lpfnCtl3dSubclassDlg == NULL || lpfnCtl3dColorChange == NULL) {
		 // error loading an address, abort
		hinstCtl3D = NULL ;
		FreeLibrary(hinstCtl3D) ;
		return ;
		} 

	// register for 3D dialox boxes
    if((*lpfnCtl3dRegister) (AfxGetInstanceHandle()))
    	(*lpfnCtl3dAutoSubclass) (AfxGetInstanceHandle()) ;
    else {
     	// if failure stop using CTL3D.DLL
		hinstCtl3D = NULL ; 
		FreeLibrary(hinstCtl3D) ;
		return ;
		} 

	SetErrorMode(ErrMode) ;         // reset the error mode
}






///////////////////////////////////////////////////////////////////////////////////////////////
/* inform the 3-D stuff about any change in the system colours
   place a call to this function in CMainWnd::OnSysColorChange()
   ie:
  		void CMainWnd::OnSysColorChange()
		{
			CFrameWnd::OnSysColorChange();
			Ctl3dColorChange() ;		// inform the 3-D stuff about the change
			-
			-
		}
*/
void Ctl3dColorChange(void)
{
		if(hinstCtl3D) (*lpfnCtl3dColorChange) () ;
}





///////////////////////////////////////////////////////////////////////////////////////////////
/* hook proceedure for use with the common dialog boxes
   place this address into the hook member of the common dialog structure
   eg:
	CPrintDialog PrintDialog ;
   		- - -
   	PrintDialog.m_pd.lpfnSetupHook = (UINT (CALLBACK *)(HWND, UINT, WPARAM, LPARAM))MakeProcInstance((FARPROC)OpenHook, AfxGetInstanceHandle());
	PrintDialog.m_pd.Flags |= PD_ENABLESETUPHOOK ;
	if(MyPrintDialog.DoModal() == IDOK) {
		- - -
		}
	FreeProcInstance(PrintDialog.m_pd.lpfnSetupHook) ;
		- - -
*/
BOOL FAR PASCAL _export OpenHook(HWND hwnd, int wm, int wParam, long lParam)
{
	BOOL r ;
	if(hinstCtl3D && wm == WM_INITDIALOG) r = (*lpfnCtl3dSubclassDlg) (hwnd, CTL3D_ALL) ;

	return FALSE;
}





///////////////////////////////////////////////////////////////////////////////////////////////
/* stop using the 3-D stuff, can be used anywhere but is normally placed
   in the application's ExitInstance function
   ie:
		BOOL CMyApp::ExitInstance()
		{
			Ctl3dUnRegister() ;
			return CWinApp::ExitInstance();
		}
*/
void Ctl3dUnRegister(void)
{
	if(hinstCtl3D) {                                    // if the CTL3D.DLL library module is loaded
		(*lpfnCtl3dUnRegister)(AfxGetInstanceHandle()); // stop using the module
		FreeLibrary(hinstCtl3D) ;                       // free the module 
		}
}